/*
 * ***** BEGIN LICENSE BLOCK *****
 * 
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2005, 2006, 2007 Zimbra, Inc.
 * 
 * The contents of this file are subject to the Yahoo! Public License
 * Version 1.0 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * 
 * ***** END LICENSE BLOCK *****
 */

/**
* @class ZaStatus 
* @contructor ZaStatus
* @param app
* @author Greg Solovyev
**/
ZaStatus = function(app) {
	ZaItem.call(this, app, "ZaStatus");
	this._init(app);	
}

ZaStatus.prototype = new ZaItem;
ZaStatus.prototype.constructor = ZaStatus;
ZaItem.loadMethods["ZaStatus"] = new Array();
ZaItem.initMethods["ZaStatus"] = new Array();

ZaStatus.A_server = "server";
ZaStatus.A_service = "service";
ZaStatus.A_timestamp = "t";
ZaStatus.PRFX_Server = "status_server";
ZaStatus.PRFX_Service = "status_service";
ZaStatus.PRFX_Time = "status_time";
ZaStatus.PRFX_Status = "status_status";

ZaStatus.loadMethod = 
function() {
	try {
		var logHost = this._app.getGlobalConfig().attrs[ZaServer.A_zimbraLogHostname];		
		//if zimbraLogHostname is set
		if (logHost) {
			var soapDoc = AjxSoapDoc.create("GetServiceStatusRequest", "urn:zimbraAdmin", null);
			var command = new ZmCsfeCommand();
			var params = new Object();
			params.soapDoc = soapDoc;	
			var resp = command.invoke(params).Body.GetServiceStatusResponse;			
			this.initFromJS(resp);
		}	
	} catch (ex) {
			this._app.getStatusViewController()._handleException(ex, "ZaStatus.loadMethod", null, false);		
	}	
}

ZaItem.loadMethods["ZaStatus"].push(ZaStatus.loadMethod);

ZaStatus.initMethod = function (app) {
	this.serverMap = new Object();
	this.statusVector = new AjxVector();
	this.id = Dwt.getNextId();
}
ZaItem.initMethods["ZaStatus"].push(ZaStatus.initMethod);

ZaStatus.prototype.initFromJS = 
function(obj) {
	if(obj.status && obj.status instanceof Array) {
		var statusArray = obj.status;
		var cnt = statusArray.length;
		var formatter = AjxDateFormat.getDateTimeInstance(AjxDateFormat.MEDIUM, AjxDateFormat.SHORT);
		for(var i=0; i < cnt; i++) {
			var serverName = statusArray[i].server;
			if(!this.serverMap[serverName]) {
				this.serverMap[serverName] = new Object();
				this.serverMap[serverName].name = serverName;
				this.serverMap[serverName].id = Dwt.getNextId();
				this.serverMap[serverName].serviceMap = null;
				this.serverMap[serverName].status = 1;
				this.statusVector.add(this.serverMap[serverName]);
			}	
			var serviceName = statusArray[i].service;
			if(serviceName) {

				if(!this.serverMap[serverName].serviceMap)
					this.serverMap[serverName].serviceMap = new Object();
					
				this.serverMap[serverName].serviceMap[serviceName] = new Object();
				this.serverMap[serverName].serviceMap[serviceName].status = statusArray[i]._content;
				this.serverMap[serverName].serviceMap[serviceName].timestamp = statusArray[i].t;
				this.serverMap[serverName].serviceMap[serviceName].time = formatter.format(new Date(Number(statusArray[i].t)*1000));
				if(this.serverMap[serverName].serviceMap[serviceName].status != 1) {
					this.serverMap[serverName].status = 0;
				}
			}
		}
	}	
}

ZaStatus.prototype.getStatusVector = 
function() {
	return this.statusVector;
}

ZaStatus.compare = function (a,b) {
	return (a.serverName < b.serverName)? -1: ((a.serverName > b.serverName)? 1: 0);
};
